package javawrapper;

/**
 * Manages analysis behaviors and data. Analysis behaviors are those whose only
 * purpose is to calculate something for output; they don't change model state.
 * 
 * Copyright: Copyright (c) Charles D. Canham 2003
 * Company: Institute of Ecosystem Studies
 * @author Lora E. Murphy
 * @version 1.0
 * 
 * <br>
 * Edit history: <br>
 * ------------------ <br>
 * October 28, 2004: Created (LEM) <br>
 * February 1, 2004: Added biomass behavior (LEM) <br>
 * March 23, 2005: Added dimension analysis behavior (LEM) <br>
 * March 28, 2005: Added bole volume behavior (LEM) <br>
 * January 11, 2006: Added tree age calculator behavior (LEM) <br>
 * March 10, 2006: Added merchantable timber value behavior (LEM) <br>
 * March 15, 2006: Added carbon value behavior (LEM) <br>
 * April 25, 2006: Added carbon value tree data member (LEM) <br>
 * November 7, 2006: Added partitioned biomass behaviors (LEM) <br>
 * September 13, 2007: Added Ripley's K calculator behavior (LEM)<br>
 * January 23, 2008: Updated partitioned biomass behavior (LEM)<br>
 * June 18, 2008: Added crown radius reporter behavior (LEM)<br>
 * December 11, 2008: Added foliar chemistry behavior (LEM)<br>
 */

public class AnalysisBehaviors extends BehaviorTypeBase {

	/** Volume calculator - a in the taper equation (also known as a0) */
	protected ModelVector mp_fTaperA = new ModelVector(
			"Taper Equation Initial Multiplier (a0)", "vo_taperA", "vo_taVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - b in the taper equation (also known as a1) */
	protected ModelVector mp_fTaperB = new ModelVector(
			"Taper Equation DBH Exponent (a1)", "vo_taperB", "vo_tbVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - c in the taper equation (also known as a2) */
	protected ModelVector mp_fTaperC = new ModelVector(
			"Taper Equation Height Exponent (a2)", "vo_taperC", "vo_tcVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - d in the taper equation (also known as b1) */
	protected ModelVector mp_fTaperD = new ModelVector(
			"Taper Equation X Exponent 1 (b1)", "vo_taperD", "vo_tdVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - f in the taper equation (also known as b2) */
	protected ModelVector mp_fTaperF = new ModelVector(
			"Taper Equation X Exponent 2 (b2)", "vo_taperF", "vo_tfVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - g in the taper equation (also known as b3) */
	protected ModelVector mp_fTaperG = new ModelVector(
			"Taper Equation X Exponent 3 (b3)", "vo_taperG", "vo_tgVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - i in the taper equation (also known as b4) */
	protected ModelVector mp_fTaperI = new ModelVector(
			"Taper Equation X Exponent 4 (b4)", "vo_taperI", "vo_tiVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - j in the taper equation (also known as b5) */
	protected ModelVector mp_fTaperJ = new ModelVector(
			"Taper Equation X Exponent 5 (b5)", "vo_taperJ", "vo_tjVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - k in the taper equation (also known as b6) */
	protected ModelVector mp_fTaperK = new ModelVector(
			"Taper Equation X Exponent 6 (b6)", "vo_taperK", "vo_tkVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - a for the diameter-outside-bark (also known as a1) */
	protected ModelVector mp_fBarkA = new ModelVector(
			"Diameter-Outside-Bark Constant (a1)", "vo_barkA", "vo_baVal", 0,
			ModelVector.FLOAT);

	/** Volume calculator - b for the diameter-outside-bark (also known as a2) */
	protected ModelVector mp_fBarkB = new ModelVector(
			"Diameter-Outside-Bark First Degree Parameter (a2)", "vo_barkB",
			"vo_bbVal", 0, ModelVector.FLOAT);

	/** Volume calculator - c for the diameter-outside-bark (also known as a3) */
	protected ModelVector mp_fBarkC = new ModelVector(
			"Diameter-Outside-Bark Second Degree Parameter (a3)", "vo_barkC",
			"vo_bcVal", 0, ModelVector.FLOAT);

	/** Dimension analysis - equation ID */
	protected ModelVector mp_iEquationID = new ModelVector(
			"Dimension Analysis Equation ID", "bi_eqID", "bi_eiVal", 0,
			ModelVector.MODEL_ENUM);

	/** Dimension analysis - DBH units */
	protected ModelVector mp_iDbhUnits = new ModelVector(
			"Dimension Analysis DBH Units", "bi_dbhUnits", "bi_duVal", 0,
			ModelVector.MODEL_ENUM);

	/** Dimension analysis - biomass units */
	protected ModelVector mp_iBiomassUnits = new ModelVector(
			"Dimension Analysis Biomass Units", "bi_biomassUnits", "bi_buVal", 0,
			ModelVector.MODEL_ENUM);

	/** Dimension analysis - whether or not to use a correction factor */
	protected ModelVector mp_iUseCorrectionFactor = new ModelVector(
			"Dimension Analysis: Use Correction Factor?", "bi_useCorrectionFactor",
			"bi_ucfVal", 0, ModelVector.MODEL_ENUM);

	/** Dimension analysis - correction factor value */
	protected ModelVector mp_fCorrectionFactor = new ModelVector(
			"Dimension Analysis Correction Factor", "bi_correctionFactorValue",
			"bi_cfvVal ", 0, ModelVector.FLOAT);

	/** Dimension analysis - a in the biomass equation */
	protected ModelVector mp_fBiomassA = new ModelVector(
			"Dimension Analysis Parameter (a)", "bi_a", "bi_aVal", 0,
			ModelVector.FLOAT);

	/** Dimension analysis - b in the biomass equation */
	protected ModelVector mp_fBiomassB = new ModelVector(
			"Dimension Analysis Parameter (b)", "bi_b", "bi_bVal", 0,
			ModelVector.FLOAT);

	/** Dimension analysis - c in the biomass equation */
	protected ModelVector mp_fBiomassC = new ModelVector(
			"Dimension Analysis Parameter (c)", "bi_c", "bi_cVal", 0,
			ModelVector.FLOAT);

	/** Dimension analysis - d in the biomass equation */
	protected ModelVector mp_fBiomassD = new ModelVector(
			"Dimension Analysis Parameter (d)", "bi_d", "bi_dVal", 0,
			ModelVector.FLOAT);

	/** Dimension analysis - e in the biomass equation */
	protected ModelVector mp_fBiomassE = new ModelVector(
			"Dimension Analysis Parameter (e)", "bi_e", "bi_eVal", 0,
			ModelVector.FLOAT);

	/** Bole volume - b0 in the volume equation */
	protected ModelVector mp_fBoleVolumeB0 = new ModelVector(
			"Bole Volume Parameter (b0)", "an_boleB0", "an_bb0Val", 0,
			ModelVector.FLOAT);

	/** Bole volume - b1 in the volume equation */
	protected ModelVector mp_fBoleVolumeB1 = new ModelVector(
			"Bole Volume Parameter (b1)", "an_boleB1", "an_bb1Val", 0,
			ModelVector.FLOAT);

	/** Bole volume - b2 in the volume equation */
	protected ModelVector mp_fBoleVolumeB2 = new ModelVector(
			"Bole Volume Parameter (b2)", "an_boleB2", "an_bb2Val", 0,
			ModelVector.FLOAT);

	/** Bole volume - b3 in the volume equation */
	protected ModelVector mp_fBoleVolumeB3 = new ModelVector(
			"Bole Volume Parameter (b3)", "an_boleB3", "an_bb3Val", 0,
			ModelVector.FLOAT);

	/** Bole volume - b4 in the volume equation */
	protected ModelVector mp_fBoleVolumeB4 = new ModelVector(
			"Bole Volume Parameter (b4)", "an_boleB4", "an_bb4Val", 0,
			ModelVector.FLOAT);

	/** Bole volume - b5 in the volume equation */
	protected ModelVector mp_fBoleVolumeB5 = new ModelVector(
			"Bole Volume Parameter (b5)", "an_boleB5", "an_bb5Val", 0,
			ModelVector.FLOAT);

	/** Bole volume - form classes */
	protected ModelVector mp_fBoleVolumeFormClasses = new ModelVector(
			"Bole Volume Form Class, 60% - 100%", "an_boleFormClasses", "an_bfcVal",
			0, ModelVector.FLOAT);

	/** Merchantable timber value - form classes */
	protected ModelVector mp_fMerchValueFormClasses = new ModelVector(
			"Merchantable Timber Value Form Class", "an_merchValueFormClasses",
			"an_mvfcVal", 0, ModelVector.MODEL_ENUM);

	/** Merchantable timber value - price per thousand board feet */
	protected ModelVector mp_fMerchValuePricePer1K = new ModelVector(
			"Merchantable Timber Value Price / 1000 Board Feet",
			"an_merchValuePricePer1KFeet", "an_mvpp1kfVal", 0, ModelVector.FLOAT);

	/** Carbon value - % of biomass that is carbon */
	protected ModelVector mp_fCarbonValuePercentBiomassCarbon = new ModelVector(
			"Carbon Value - Carbon Amount of Biomass (0-100%)",
			"an_carbonPercentBiomassCarbon", "an_cpbcVal", 0, ModelVector.FLOAT);

	/** Partitioned DBH biomass - Leaf DBH-biomass "a" */
	protected ModelVector mp_fPartBioDbhLeafA = new ModelVector(
			"Partitioned DBH Biomass - Leaf Slope (a)", "an_partBioDbhLeafA",
			"an_pbdlaVal", 0, ModelVector.FLOAT);

	/** Partitioned DBH biomass - Leaf DBH-biomass "b" */
	protected ModelVector mp_fPartBioDbhLeafB = new ModelVector(
			"Partitioned DBH Biomass - Leaf Intercept (b)", "an_partBioDbhLeafB",
			"an_pbdlbVal", 0, ModelVector.FLOAT);

	/** Partitioned DBH biomass - Branch DBH-biomass "a" */
	protected ModelVector mp_fPartBioDbhBranchA = new ModelVector(
			"Partitioned DBH Biomass - Branch Slope (a)", "an_partBioDbhBranchA",
			"an_pbdbraVal", 0, ModelVector.FLOAT);

	/** Partitioned DBH biomass - Branch DBH-biomass "b" */
	protected ModelVector mp_fPartBioDbhBranchB = new ModelVector(
			"Partitioned DBH Biomass - Branch Intercept (b)", "an_partBioDbhBranchB",
			"an_pbdbrbVal", 0, ModelVector.FLOAT);

	/** Partitioned DBH biomass - Bole DBH-biomass "a" */
	protected ModelVector mp_fPartBioDbhBoleA = new ModelVector(
			"Partitioned DBH Biomass - Bole Slope (a)", "an_partBioDbhBoleA",
			"an_pbdboaVal", 0, ModelVector.FLOAT);

	/** Partitioned DBH biomass - Bole DBH-biomass "b" */
	protected ModelVector mp_fPartBioDbhBoleB = new ModelVector(
			"Partitioned DBH Biomass - Bole Intercept (b)", "an_partBioDbhBoleB",
			"an_pbdbobVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - foliar weight "a" */
  protected ModelVector mp_fFoliarChemA  = new ModelVector(
      "Foliar Chemistry - Foliar Weight (a)", "an_foliarChemWeightA",
      "an_fcwaVal", 0, ModelVector.FLOAT);
  
  /** Foliar chemistry - foliar weight "b" */
  protected ModelVector mp_fFoliarChemB  = new ModelVector(
      "Foliar Chemistry - Foliar Weight (b)", "an_foliarChemWeightB",
      "an_fcwbVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - N concentration */
	protected ModelVector mp_fFoliarChemN  = new ModelVector(
			"Foliar Chemistry - N Concentration", "an_foliarChemN",
			"an_fcnVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - P concentration */
	protected ModelVector mp_fFoliarChemP  = new ModelVector(
			"Foliar Chemistry - P Concentration", "an_foliarChemP",
			"an_fcpVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - lignin concentration */
	protected ModelVector mp_fFoliarChemLignin  = new ModelVector(
			"Foliar Chemistry - Lignin Concentration", "an_foliarChemLignin",
			"an_fclVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - fiber concentration */
	protected ModelVector mp_fFoliarChemFiber  = new ModelVector(
			"Foliar Chemistry - Fiber Concentration", "an_foliarChemFiber",
			"an_fcfVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - cellulose concentration */
	protected ModelVector mp_fFoliarChemCellulose  = new ModelVector(
			"Foliar Chemistry - Cellulose Concentration", "an_foliarChemCellulose",
			"an_fccVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - tannins concentration */
	protected ModelVector mp_fFoliarChemTannins  = new ModelVector(
			"Foliar Chemistry - Tannins Concentration", "an_foliarChemTannins",
			"an_fctVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - phenolics concentration */
	protected ModelVector mp_fFoliarChemPhenolics  = new ModelVector(
			"Foliar Chemistry - Phenolics Concentration", "an_foliarChemPhenolics",
			"an_fcphVal", 0, ModelVector.FLOAT);
	
	/** Foliar chemistry - SLA concentration */
	protected ModelVector mp_fFoliarChemSLA  = new ModelVector(
			"Foliar Chemistry - SLA Concentration", "an_foliarChemSLA",
			"an_fcsVal", 0, ModelVector.FLOAT);
	
	/** Partitioned height biomass - Leaf height-biomass "a" */
	protected ModelVector mp_fPartBioHeightLeafA = new ModelVector(
			"Partitioned Palm Biomass - Leaf Slope (a)", "an_partBioHeightLeafA",
			"an_pbhlaVal", 0, ModelVector.FLOAT);

	/** Partitioned height biomass - Leaf height-biomass "b" */
	protected ModelVector mp_fPartBioHeightLeafB = new ModelVector(
			"Partitioned Palm Biomass - Leaf Intercept (b)", "an_partBioHeightLeafB",
			"an_pbhlbVal", 0, ModelVector.FLOAT);

	/** Partitioned height biomass - Bole height-biomass "a" */
	protected ModelVector mp_fPartBioHeightBoleA = new ModelVector(
			"Partitioned Palm Biomass - Bole Slope (a)", "an_partBioHeightBoleA",
			"an_pbhboaVal", 0, ModelVector.FLOAT);

	/** Partitioned height biomass - Bole height-biomass "b" */
	protected ModelVector mp_fPartBioHeightBoleB = new ModelVector(
			"Partitioned Palm Biomass - Bole Intercept (b)", "an_partBioHeightBoleB",
			"an_pbhbobVal", 0, ModelVector.FLOAT);

	/**
	 * Volume calculator - stump height, in cm. Point at which to start summing
	 * trunk volume
	 */
	protected ModelFloat m_fStumpHeight = new ModelFloat(0,
			"Height to Begin Calculating Trunk Volume, in cm", "vo_stumpHeight");

	/**
	 * Volume calculator - minimum usable diameter, in cm. Point at which to stop
	 * summing trunk volume
	 */
	protected ModelFloat m_fMinUsableDiam = new ModelFloat(0,
			"Minimum Trunk Diameter for Volume Calculations, in cm",
			"vo_minUsableDiam");

	/** Volume calculator - Length of tree trunk volume segments, in m */
	protected ModelFloat m_fSegmentLength = new ModelFloat(0,
			"Trunk Segment Length for Volume Calculations, in m", "vo_segmentLength");

	/** Carbon value - value of metric ton of carbon */
	protected ModelFloat m_fCarbonValueCarbonPrice = new ModelFloat(0,
			"Carbon Value - Price Per Metric Ton Carbon",
			"an_carbonPricePerMetricTonCarbon");

	/** Ripley's K - maximum distance */
	protected ModelFloat m_fRipleysKMaxDistance = new ModelFloat(50,
			"Ripley's K - Maximum Distance (m)", "an_RipleysKMaxDistance");

	/** Ripley's K - distance increment */
	protected ModelFloat m_fRipleysKDistanceInc = new ModelFloat(1,
			"Ripley's K - Distance Increment (m)", "an_RipleysKDistanceInc");
	
	/** Relative neighborhood density (Condit's omega) - maximum distance */
	protected ModelFloat m_fConditsOmegaMaxDistance = new ModelFloat(50,
			"Relative Neighborhood Density - Maximum Distance (m)", "an_ConditsOmegaMaxDistance");

	/** Relative neighborhood density (Condit's omega) - distance increment */
	protected ModelFloat m_fConditsOmegaDistanceInc = new ModelFloat(1,
			"Relative Neighborhood Density - Distance Increment (m)", "an_ConditsOmegaDistanceInc");

	/**
	 * Constructor.
	 * 
	 * @param oManager
	 *          GUIManager object.
	 * @throws ModelException
	 *           passed through from called functions.
	 */
	public AnalysisBehaviors(GUIManager oManager) throws ModelException {
		super(oManager, "Analysis", "analysis");

		int iIndex = 0;

		m_sHelpID = "data.analysis_behaviors";

		// Make our child behaviors
		mp_oChildBehaviors = new Behavior[13];

		// Carbon value behavior - can be edited automatically, must
		// have trees
		mp_oChildBehaviors[iIndex] = new Behavior("Carbon Value",
				"Carbon Value Calculator", "Carbon Value Calculator", (float) 1);
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Mg Carbon", "Carbon", DataMember.FLOAT));
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex]
				.AddRequiredData(mp_fCarbonValuePercentBiomassCarbon);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fCarbonValueCarbonPrice);
		iIndex++;
		
	  // Crown radius reporter behavior - can be edited automatically, must
    // have trees
/*    mp_oChildBehaviors[iIndex] = new Behavior("Crown Radius Reporter",
        "Crown Radius Reporter", "Crown Radius Reporter", (float) 1);
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
        "Crown Radius", "Crown_Radius", DataMember.FLOAT));
    mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SNAG, true);
    iIndex++;*/

		// Dimension analysis behavior - can be edited automatically, must have
		// trees
		mp_oChildBehaviors[iIndex] = new Behavior("tree biomass calculator",
				"Dimension Analysis", "tree biomass calculator", (float) 2);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		// Data members
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Tree Biomass", "Biomass", DataMember.FLOAT));
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_iEquationID);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_iDbhUnits);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_iBiomassUnits);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_iUseCorrectionFactor);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCorrectionFactor);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiomassA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiomassB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiomassC);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiomassD);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBiomassE);
		iIndex++;
		
	  // Foliar chemistry behavior - can be edited automatically, must
    // have trees
    mp_oChildBehaviors[iIndex] = new Behavior("Foliar Chemistry",
        "Foliar Chemistry", "Foliar Chemistry", (float) 1);
    mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemN);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemP);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemLignin);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemFiber);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemCellulose);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemTannins);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemPhenolics);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fFoliarChemSLA);
    iIndex++;

		// Merchantable timber value behavior - can be edited automatically, must
		// have trees
		mp_oChildBehaviors[iIndex] = new Behavior("Merchantable Timber Value",
				"Merchantable Timber Value", "Merch Value Calculator", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		// Data members
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Merchantable Value", "Merch Val", DataMember.FLOAT));
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMerchValueFormClasses);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMerchValuePricePer1K);
		iIndex++;

		// Partitioned DBH Biomass behavior - can be edited automatically, must
		// have trees
		mp_oChildBehaviors[iIndex] = new Behavior("Partitioned DBH Biomass",
				"Partitioned DBH Biomass", "Partitioned DBH Biomass", (float) 2);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhLeafA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhLeafB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBranchA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBranchB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBoleA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBoleB);
		
		iIndex++;

		// Partitioned Palm Biomass behavior - can be edited automatically, must
		// have trees
		mp_oChildBehaviors[iIndex] = new Behavior("Partitioned Height Biomass",
				"Partitioned Palm Biomass", "Partitioned Height Biomass", (float) 2);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightLeafA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightLeafB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightBoleA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightBoleB);
		iIndex++;
		
    //Relative neighborhood density (Condit's omega) calculator behavior - 
		//doesn't need trees
		mp_oChildBehaviors[iIndex] = new Behavior("Condit's Omega",
				"Relative Neighborhood Density Calculator", "Condit's Omega", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fConditsOmegaMaxDistance);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fConditsOmegaDistanceInc);
		iIndex++;

		// Ripley's K Calculator behavior - doesn't need trees
		mp_oChildBehaviors[iIndex] = new Behavior("Ripley's K",
				"Ripley's K Calculator", "Ripley's K", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = false;
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fRipleysKMaxDistance);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fRipleysKDistanceInc);
		iIndex++;

		// Partitioned DBH Biomass behavior - can be edited automatically, must
		// have trees
		mp_oChildBehaviors[iIndex] = new Behavior(
				"Storm Killed Partitioned DBH Biomass",
				"Storm Killed Partitioned DBH Biomass",
				"Storm Killed Partitioned DBH Biomass", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SNAG, true);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEED, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SAPLING, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.ADULT, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhLeafA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhLeafB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBranchA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBranchB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBoleA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioDbhBoleB);
		iIndex++;

		// Partitioned Palm Biomass behavior - can be edited automatically, must
		// have trees
		mp_oChildBehaviors[iIndex] = new Behavior(
				"Storm Killed Partitioned Height Biomass",
				"Storm Killed Partitioned Palm Biomass",
				"Storm Killed Partitioned Height Biomass", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEED, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SAPLING, false);
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.ADULT, false);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightLeafA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightLeafB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightBoleA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPartBioHeightBoleB);
		iIndex++;

		// Tree age calculator behavior - can be edited automatically, must have
		// trees
		mp_oChildBehaviors[iIndex] = new Behavior("Tree Age", "Tree Age",
				"Tree Age Calculator", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SNAG, false);
		// Data members
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Tree Age", "Tree Age", DataMember.INTEGER));
		// No parameters
		iIndex++;

		// Dimension analysis behavior - can be edited automatically, must have
		// trees
		mp_oChildBehaviors[iIndex] = new Behavior("bole volume",
				"Tree Bole Volume Calculator", "tree bole volume calculator", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		// Data members
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Tree Bole Volume", "Bole Vol", DataMember.FLOAT));
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeFormClasses);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeB0);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeB1);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeB2);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeB3);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeB4);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBoleVolumeB5);
		iIndex++;

		// Volume calculator behavior - can be edited automatically, must have trees
		mp_oChildBehaviors[iIndex] = new Behavior("tree volume calculator",
				"Tree Volume Calculator", "tree volume calculator", (float) 1);
		mp_oChildBehaviors[iIndex].m_bMustHaveTrees = true;
		mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
		// Data members
		mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember(
				"Tree Volume", "Volume", DataMember.FLOAT));
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperC);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperD);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperF);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperG);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperI);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperJ);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTaperK);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBarkA);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBarkB);
		mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBarkC);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fMinUsableDiam);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fSegmentLength);
		mp_oChildBehaviors[iIndex].AddRequiredData(m_fStumpHeight);
		iIndex++;

		mp_oAllData.add(mp_fTaperA);
		mp_oAllData.add(mp_fTaperB);
		mp_oAllData.add(mp_fTaperC);
		mp_oAllData.add(mp_fTaperD);
		mp_oAllData.add(mp_fTaperF);
		mp_oAllData.add(mp_fTaperG);
		mp_oAllData.add(mp_fTaperI);
		mp_oAllData.add(mp_fTaperJ);
		mp_oAllData.add(mp_fTaperK);
		mp_oAllData.add(mp_fBarkA);
		mp_oAllData.add(mp_fBarkB);
		mp_oAllData.add(mp_fBarkC);
		mp_oAllData.add(m_fMinUsableDiam);
		mp_oAllData.add(m_fSegmentLength);
		mp_oAllData.add(m_fStumpHeight);
		mp_oAllData.add(mp_iEquationID);
		mp_oAllData.add(mp_iDbhUnits);
		mp_oAllData.add(mp_iBiomassUnits);
		mp_oAllData.add(mp_iUseCorrectionFactor);
		mp_oAllData.add(mp_fCorrectionFactor);
		mp_oAllData.add(mp_fBiomassA);
		mp_oAllData.add(mp_fBiomassB);
		mp_oAllData.add(mp_fBiomassC);
		mp_oAllData.add(mp_fBiomassD);
		mp_oAllData.add(mp_fBiomassE);
		mp_oAllData.add(mp_fBoleVolumeFormClasses);
		mp_oAllData.add(mp_fBoleVolumeB0);
		mp_oAllData.add(mp_fBoleVolumeB1);
		mp_oAllData.add(mp_fBoleVolumeB2);
		mp_oAllData.add(mp_fBoleVolumeB3);
		mp_oAllData.add(mp_fBoleVolumeB4);
		mp_oAllData.add(mp_fBoleVolumeB5);
		mp_oAllData.add(mp_fMerchValueFormClasses);
		mp_oAllData.add(mp_fMerchValuePricePer1K);
		mp_oAllData.add(mp_fCarbonValuePercentBiomassCarbon);
		mp_oAllData.add(m_fCarbonValueCarbonPrice);
		mp_oAllData.add(mp_fPartBioDbhLeafA);
		mp_oAllData.add(mp_fPartBioDbhLeafB);
		mp_oAllData.add(mp_fPartBioDbhBranchA);
		mp_oAllData.add(mp_fPartBioDbhBranchB);
		mp_oAllData.add(mp_fPartBioDbhBoleA);
		mp_oAllData.add(mp_fPartBioDbhBoleB);
		mp_oAllData.add(mp_fFoliarChemA);
		mp_oAllData.add(mp_fFoliarChemB);
		mp_oAllData.add(mp_fFoliarChemN);
		mp_oAllData.add(mp_fFoliarChemP);
		mp_oAllData.add(mp_fFoliarChemLignin);
		mp_oAllData.add(mp_fFoliarChemFiber);
		mp_oAllData.add(mp_fFoliarChemCellulose);
		mp_oAllData.add(mp_fFoliarChemTannins);
		mp_oAllData.add(mp_fFoliarChemPhenolics);
		mp_oAllData.add(mp_fFoliarChemSLA);
		mp_oAllData.add(mp_fPartBioHeightLeafA);
		mp_oAllData.add(mp_fPartBioHeightLeafB);
		mp_oAllData.add(mp_fPartBioHeightBoleA);
		mp_oAllData.add(mp_fPartBioHeightBoleB);
		mp_oAllData.add(m_fRipleysKMaxDistance);
		mp_oAllData.add(m_fRipleysKDistanceInc);
		mp_oAllData.add(m_fConditsOmegaMaxDistance);
		mp_oAllData.add(m_fConditsOmegaDistanceInc);
	}

	/**
	 * Sets up the enum parameters and grids.
	 * 
	 * @param oPop TreePopulation object.
	 * @throws javawrapper.ModelException Doesn't throw this exception.
	 */
	public void DoSetup(TreePopulation oPop) throws javawrapper.ModelException {
		ModelEnum oVal;
		int iNumSpecies = oPop.GetNumberOfSpecies(), i;

		// Set up the equation ID enums vector - values display in reverse order
		if (mp_iEquationID.GetValue().size() == 0) {
			for (i = 0; i < iNumSpecies; i++) {
				oVal = new ModelEnum(new int[] { 9, 8, 7, 6, 5, 4, 3, 2, 1 },
						new String[] { "9", "8", "7", "6", "5", "4", "3", "2", "1" }, "",
						"");
				oVal.SetValue(1);
				mp_iEquationID.GetValue().add(oVal);
			}
		} else {
			// We may have added species - add enums if appropriate
			for (i = 0; i < iNumSpecies; i++) {
				if (null == mp_iEquationID.GetValue().elementAt(i)) {
					oVal = new ModelEnum(new int[] { 9, 8, 7, 6, 5, 4, 3, 2, 1 },
							new String[] { "9", "8", "7", "6", "5", "4", "3", "2", "1" }, "",
							"");
					oVal.SetValue(1);
					mp_iEquationID.GetValue().remove(i);
					mp_iEquationID.GetValue().add(i, oVal);
				}
			}
		}

		if (mp_iDbhUnits.GetValue().size() == 0) {
			// Set up the DBH units enums vector - values display in reverse order
			mp_iDbhUnits.GetValue().clear();
			for (i = 0; i < iNumSpecies; i++) {
				oVal = new ModelEnum(new int[] { 2, 1, 0 }, new String[] { "in", "cm",
						"mm" }, "", "");
				oVal.SetValue(0);
				mp_iDbhUnits.GetValue().add(oVal);
			}
		} else {
			// We may have added species - add enums if appropriate
			for (i = 0; i < iNumSpecies; i++) {
				if (null == mp_iDbhUnits.GetValue().elementAt(i)) {
					oVal = new ModelEnum(new int[] { 2, 1, 0 }, new String[] { "in",
							"cm", "mm" }, "", "");
					oVal.SetValue(0);
					mp_iDbhUnits.GetValue().remove(i);
					mp_iDbhUnits.GetValue().add(i, oVal);
				}
			}
		}

		// Set up the biomass units enums vector - values display in reverse order
		if (mp_iBiomassUnits.GetValue().size() == 0) {
			mp_iBiomassUnits.GetValue().clear();
			for (i = 0; i < iNumSpecies; i++) {
				oVal = new ModelEnum(new int[] { 2, 1, 0 }, new String[] { "lb", "kg",
						"g" }, "", "");
				mp_iBiomassUnits.GetValue().add(oVal);
			}
		} else {
			// We may have added species - add enums if appropriate
			for (i = 0; i < iNumSpecies; i++) {
				if (null == mp_iBiomassUnits.GetValue().elementAt(i)) {
					oVal = new ModelEnum(new int[] { 2, 1, 0 }, new String[] { "lb",
							"kg", "g" }, "", "");
					mp_iBiomassUnits.GetValue().remove(i);
					mp_iBiomassUnits.GetValue().add(i, oVal);
				}
			}
		}

		// Set up the use correction factor enums vector - default all to false
		if (mp_iUseCorrectionFactor.GetValue().size() == 0) {
			mp_iUseCorrectionFactor.GetValue().clear();
			for (i = 0; i < iNumSpecies; i++) {
				oVal = new ModelEnum(new int[] { 1, 0 },
						new String[] { "true", "false" }, "", "");
				oVal.SetValue(0);
				mp_iUseCorrectionFactor.GetValue().add(oVal);
			}
		} else {
			// We may have added species - add enums if appropriate
			for (i = 0; i < iNumSpecies; i++) {
				if (null == mp_iUseCorrectionFactor.GetValue().elementAt(i)) {
					oVal = new ModelEnum(new int[] { 1, 0 }, new String[] { "true",
							"false" }, "", "");
					oVal.SetValue(0);
					mp_iUseCorrectionFactor.GetValue().remove(i);
					mp_iUseCorrectionFactor.GetValue().add(i, oVal);
				}
			}
		}

		// Set up the merchantable timber value form class enums vector - values
		// display in reverse order
		if (mp_fMerchValueFormClasses.GetValue().size() == 0) {
			mp_fMerchValueFormClasses.GetValue().clear();
			for (i = 0; i < iNumSpecies; i++) {
				oVal = new ModelEnum(new int[] { 85, 84, 81, 80, 79, 78 },
						new String[] { "85", "84", "81", "80", "79", "78" }, "", "");
				mp_fMerchValueFormClasses.GetValue().add(oVal);
			}
		} else {
			// We may have added species - add enums if appropriate
			for (i = 0; i < iNumSpecies; i++) {
				if (null == mp_fMerchValueFormClasses.GetValue().elementAt(i)) {
					oVal = new ModelEnum(new int[] { 85, 84, 81, 80, 79, 78 },
							new String[] { "85", "84", "81", "80", "79", "78" }, "", "");
					mp_fMerchValueFormClasses.GetValue().remove(i);
					mp_fMerchValueFormClasses.GetValue().add(i, oVal);
				}
			}
		}

		// Default all correction factors to 1
		if (mp_fCorrectionFactor.GetValue().size() == 0) {
			for (i = 0; i < iNumSpecies; i++) {
				mp_fCorrectionFactor.GetValue().add(new Float(1.0));
			}
		}

		// Set up the value grid for merchantable timber and carbon value
		DataMember[] p_oDataMembers = new DataMember[iNumSpecies];
		Plot p_oPlot = m_oManager.GetPlot();
		Grid oNewGrid;
		// mp_oGrids = new Grid[2];

		String sGridName = "Merchantable Timber Value";

		// The accessible data members are a value for each species
		for (i = 0; i < iNumSpecies; i++) {

			p_oDataMembers[i] = new DataMember("Value for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "value_" + i,
					DataMember.FLOAT);
		}

		// Create our grid
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, p_oPlot
				.GetPlotXLength(), p_oPlot.GetPlotYLength());

		// Assign the grid to the merchantable value behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Merchantable Timber Value").AddGrid(oNewGrid);

		// Carbon value grid
		sGridName = "Carbon Value";
		p_oDataMembers = new DataMember[2 * iNumSpecies];

		// The accessible data members are a carbon amount and carbon value for
		// each species
		for (i = 0; i < iNumSpecies; i++) {

			p_oDataMembers[i] = new DataMember("Mg of Carbon for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "carbon_" + i,
					DataMember.FLOAT);
		}

		for (i = 0; i < iNumSpecies; i++) {

			p_oDataMembers[i + iNumSpecies] = new DataMember("Carbon Value for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "value_" + i,
					DataMember.FLOAT);
		}

		// Create our grid
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, 
				p_oPlot.GetPlotXLength(), p_oPlot.GetPlotYLength());

		// Assign the grid to the carbon value behavior
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Carbon Value").AddGrid(oNewGrid);

		// Partitioned biomass grid
		sGridName = "Partitioned Biomass";
		p_oDataMembers = new DataMember[5 * iNumSpecies];
		int iStart = 0;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Leaf Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "leaf_" + i,
					DataMember.FLOAT);
		}
		iStart = iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Bole Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "bole_" + i,
					DataMember.FLOAT);
		}
		iStart = 2 * iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Branch Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "branch_" + i,
					DataMember.FLOAT);
		}
		iStart = 3 * iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Leaf Palm Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "hleaf_" + i,
					DataMember.FLOAT);
		}
		iStart = 4 * iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Bole Palm Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "hbole_" + i,
					DataMember.FLOAT);
		}
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Partitioned DBH Biomass").AddGrid(oNewGrid);
		GetBehaviorByKey("Partitioned Height Biomass").AddGrid(oNewGrid);
		
	  // Foliar chemistry grid
    sGridName = "Foliar Chemistry";
    p_oDataMembers = new DataMember[8 * iNumSpecies];
    iStart = 0;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg N for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "N_" + i,
          DataMember.FLOAT);
    }
    iStart = iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg P for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "P_" + i,
          DataMember.FLOAT);
    }
    iStart = 2 * iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg SLA for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "SLA_" + i,
          DataMember.FLOAT);
    }
    iStart = 3 * iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg Lignin for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "lignin_" + i,
          DataMember.FLOAT);
    }
    iStart = 4 * iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg Fiber for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "fiber_" + i,
          DataMember.FLOAT);
    }
    iStart = 5 * iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg Cellulose for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "cellulose_" + i,
          DataMember.FLOAT);
    }
    iStart = 6 * iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg Tannins for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "tannins_" + i,
          DataMember.FLOAT);
    }
    iStart = 7 * iNumSpecies;
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i + iStart] = new DataMember("Kg Phenolics for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "phenolics_" + i,
          DataMember.FLOAT);
    }
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Foliar Chemistry").AddGrid(oNewGrid);

		// Storm killed partitioned biomass grid
		sGridName = "Storm Killed Partitioned Biomass";
		p_oDataMembers = new DataMember[5 * iNumSpecies];
		iStart = 0;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Leaf Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "leaf_" + i,
					DataMember.FLOAT);
		}
		iStart = iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Bole Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "bole_" + i,
					DataMember.FLOAT);
		}
		iStart = 2 * iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Branch Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "branch_" + i,
					DataMember.FLOAT);
		}
		iStart = 3 * iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Leaf Palm Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "hleaf_" + i,
					DataMember.FLOAT);
		}
		iStart = 4 * iNumSpecies;
		for (i = 0; i < iNumSpecies; i++) {
			p_oDataMembers[i + iStart] = new DataMember("Mg Bole Palm Biomass for "
					+ oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "hbole_" + i,
					DataMember.FLOAT);
		}
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Storm Killed Partitioned DBH Biomass").AddGrid(oNewGrid);
		GetBehaviorByKey("Storm Killed Partitioned Height Biomass").AddGrid(
				oNewGrid);

		// Ripley's K Grid
		sGridName = "Ripley's K";
		int iNumIncs = (int) java.lang.Math.ceil(m_fRipleysKMaxDistance.GetValue()
				/ m_fRipleysKDistanceInc.GetValue()), j;
		p_oDataMembers = new DataMember[(iNumIncs * (iNumSpecies + 1))+2];
		
		p_oDataMembers[0] = new DataMember("Distance Inc", "inc", DataMember.FLOAT);
		p_oDataMembers[1] = new DataMember("Max Distance", "dist", DataMember.FLOAT);
		
		iStart = 2;
		for (j = 0; j < iNumIncs; j++) {
			p_oDataMembers[iStart] = new DataMember("All Species Dist " + j + 
					"K Value", j + "_" + iNumSpecies, DataMember.FLOAT);
			iStart++;
		}
		for (i = 0; i < iNumSpecies; i++) {
			for (j = 0; j < iNumIncs; j++) {
				p_oDataMembers[iStart] = new DataMember(oPop.GetSpeciesNameFromCode(i)
						.replace('_', ' ')
						+ " Dist " + j + "K Value", j + "_" + i, DataMember.FLOAT);
				iStart++;
			}
		}
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, p_oPlot
				.GetPlotXLength(), p_oPlot.GetPlotYLength());
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Ripley's K").AddGrid(oNewGrid);
		
    //Condit's Omega Grid (Relative Neighborhood Density)
		sGridName = "Relative Neighborhood Density";
		iNumIncs = (int) java.lang.Math.ceil(m_fConditsOmegaMaxDistance.GetValue()
				/ m_fConditsOmegaDistanceInc.GetValue());
		p_oDataMembers = new DataMember[(iNumIncs * iNumSpecies)+2];
		
		p_oDataMembers[0] = new DataMember("Distance Inc", "inc", DataMember.FLOAT);
		p_oDataMembers[1] = new DataMember("Max Distance", "dist", DataMember.FLOAT);
		
		iStart = 2;
		for (i = 0; i < iNumSpecies; i++) {
			for (j = 0; j < iNumIncs; j++) {
				p_oDataMembers[iStart] = new DataMember(oPop.GetSpeciesNameFromCode(i)
						.replace('_', ' ')
						+ " Dist " + j + " Omega", j + "_" + i, DataMember.FLOAT);
				iStart++;
			}
		}
		oNewGrid = new Grid(sGridName, p_oDataMembers, null, p_oPlot
				.GetPlotXLength(), p_oPlot.GetPlotYLength());
		oNewGrid = m_oManager.AddGrid(oNewGrid);
		GetBehaviorByKey("Condit's Omega").AddGrid(oNewGrid);
	}

	/**
	 * Validates the data before writing to a parameter file.
	 * 
	 * @throws ModelException
	 *           if:
	 *           <ul>
	 *           <li>Volume calculator is enabled, and either m_fStumpHeight or
	 *           m_fMinUsableDiam, is less than 0, or m_fSegmentLength is less
	 *           than or equal to 0;</li>
	 *           <li>Bole volume calculator is enabled, and a form class is not
	 *           between 60 and 100.</li>
	 *           <li>Carbon value calculator is enabled, and the percent of
	 *           biomass that is carbon is not between 0 and 100.</li>
	 *           <li>Carbon value calculator is enabled, and the dimension
	 *           analysis behavior is not enabled.</li>
	 *           <li>Ripley's K Calculator behavior is enabled and either the
	 *           increment or the max distance are less than or equal to zero, or
	 *           if the increment is less than the max distance</li>
	 *           </ul>
	 * @param oPop
	 *          Not used.
	 */
	public void ValidateData(TreePopulation oPop) throws ModelException {

		if (!AnyBehaviorsEnabled()) {
			return;
		}

		// Validate volume calculator parameters
		if (GetBehaviorByKey("tree volume calculator").m_bIsEnabled) {
			MakeSureGreaterThanEqualTo(m_fMinUsableDiam, 0);
			MakeSureGreaterThan(m_fSegmentLength, 0);
			MakeSureGreaterThanEqualTo(m_fStumpHeight, 0);
			int iNumSpecies = oPop.GetNumberOfSpecies();
			MakeSureRightSize(mp_fTaperA, iNumSpecies);
			MakeSureRightSize(mp_fTaperB, iNumSpecies);
			MakeSureRightSize(mp_fTaperC, iNumSpecies);
			MakeSureRightSize(mp_fTaperD, iNumSpecies);
			MakeSureRightSize(mp_fTaperF, iNumSpecies);
			MakeSureRightSize(mp_fTaperG, iNumSpecies);
			MakeSureRightSize(mp_fTaperI, iNumSpecies);
			MakeSureRightSize(mp_fTaperJ, iNumSpecies);
			MakeSureRightSize(mp_fTaperK, iNumSpecies);
			MakeSureRightSize(mp_fBarkA, iNumSpecies);
			MakeSureRightSize(mp_fBarkB, iNumSpecies);
			MakeSureRightSize(mp_fBarkC, iNumSpecies);
		}

		// Validate bole volume parameters - LEM 03-28-05
		Behavior oBeh = GetBehaviorByKey("bole volume");
		if (oBeh.m_bIsEnabled) {
			int iNumSpecies = oPop.GetNumberOfSpecies();
			MakeSureRightSize(mp_fBoleVolumeB0, iNumSpecies);
			MakeSureRightSize(mp_fBoleVolumeB1, iNumSpecies);
			MakeSureRightSize(mp_fBoleVolumeB2, iNumSpecies);
			MakeSureRightSize(mp_fBoleVolumeB3, iNumSpecies);
			MakeSureRightSize(mp_fBoleVolumeB4, iNumSpecies);
			MakeSureRightSize(mp_fBoleVolumeB5, iNumSpecies);
			MakeSureRightSize(mp_fBoleVolumeFormClasses, iNumSpecies);
			boolean[] p_bApplies = oBeh.GetWhichSpeciesUsed(oPop);
			MakeSureAllAreBounded(mp_fBoleVolumeFormClasses, p_bApplies, 60, 100);
		}

		// Validate carbon value parameters - LEM 03-15-06
		oBeh = GetBehaviorByKey("Carbon Value");
		if (oBeh.m_bIsEnabled) {
			boolean[] p_bApplies = oBeh.GetWhichSpeciesUsed(oPop);

			// Make sure all values in percent of biomass that is carbon are
			// between 0 and 100
			MakeSureAllAreBounded(mp_fCarbonValuePercentBiomassCarbon, p_bApplies, 0,
					100);

			// Make sure that all trees to which this behavior are applied also have
			// dimension analysis applied
			Behavior oDimAnBeh = GetBehaviorByKey("tree biomass calculator");
			if (false == oDimAnBeh.m_bIsEnabled) {
				throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "If you "
						+ "want to use the behavior \"" + oBeh.GetDescriptor()
						+ "\", you must " + "also use the behavior \""
						+ oDimAnBeh.GetDescriptor() + "\"."));
			}
			SpeciesTypeCombo oCarbonCombo, oDimAnCombo;
			int i, j, iNumCarbonCombos = oBeh.GetNumberOfCombos(), iNumDimAnCombos = oDimAnBeh
					.GetNumberOfCombos();
			boolean bFound;
			for (i = 0; i < iNumCarbonCombos; i++) {
				bFound = false;
				oCarbonCombo = oBeh.GetSpeciesTypeCombo(i);
				for (j = 0; j < iNumDimAnCombos; j++) {
					oDimAnCombo = oDimAnBeh.GetSpeciesTypeCombo(j);
					if (oCarbonCombo.equals(oDimAnCombo)) {
						bFound = true;
						break;
					}
				}
				if (false == bFound) {
					throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
							"All trees " + "that use the behavior \"" + oBeh.GetDescriptor()
									+ "\" must also use the " + oDimAnBeh.GetDescriptor()
									+ "\" behavior."));
				}
			}
		}

		// Validate Ripley's K calculator parameters
		oBeh = GetBehaviorByKey("Ripley's K");
		if (oBeh.m_bIsEnabled) {
			MakeSureGreaterThan(m_fRipleysKDistanceInc, 0);
			MakeSureGreaterThan(m_fRipleysKMaxDistance, 0);
			if (m_fRipleysKDistanceInc.GetValue() > m_fRipleysKMaxDistance.GetValue()) {
				throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The value for "
						+ m_fRipleysKMaxDistance.GetDescriptor()
						+ " must be greater than the value for "
						+ m_fRipleysKDistanceInc.GetDescriptor() + "."));
			}

			// Now see if we need to change the number of Ripley's K grid data members
			// since it depends on changes the user may have made to parameters
			int iNumIncs = (int) java.lang.Math.ceil(m_fRipleysKMaxDistance
					.GetValue()	/ m_fRipleysKDistanceInc.GetValue()), 
					iNumSpecies = oPop.GetNumberOfSpecies(), iStart, i, j, k;
			Grid oGrid = m_oManager.GetGridByName("Ripley's K");
			DataMember[] p_oDataMembers = oGrid.GetDataMembers();
			if (p_oDataMembers.length != ((iNumIncs * (iNumSpecies + 1)) + 2)) {
				p_oDataMembers = new DataMember[(iNumIncs * (iNumSpecies + 1))+2];
				p_oDataMembers[0] = new DataMember("Distance Inc", "inc", DataMember.FLOAT);
				p_oDataMembers[1] = new DataMember("Max Distance", "dist", DataMember.FLOAT);
				iStart = 2;
				for (j = 0; j < iNumIncs; j++) {
					p_oDataMembers[iStart] = new DataMember(
							"All Species Dist " + j + "K Value", j + "_" + iNumSpecies,
							DataMember.FLOAT);
					iStart++;
				}
				for (i = 0; i < iNumSpecies; i++) {
					for (j = 0; j < iNumIncs; j++) {
						p_oDataMembers[iStart] = new DataMember(oPop
								.GetSpeciesNameFromCode(i).replace('_', ' ')
								+ " Dist " + j + "K Value", j + "_" + i, DataMember.FLOAT);
						iStart++;
					}
				}
				
				oGrid.SetDataMembers(p_oDataMembers);
				
				//Go to the output behavior - if Ripley's K is a grid, erase any
				//data members that may have been removed
				OutputBehaviors oOutput = m_oManager.GetOutputBehaviors();
				DetailedGridSettings oSettings;
				DataMember oMember;
				boolean bFound;
				for (i = 0; i < oOutput.mp_oDetailedGridSaveSettings.size(); i++) {
					oSettings = (DetailedGridSettings) oOutput.mp_oDetailedGridSaveSettings.elementAt(i);
					if (oSettings.GetName().equals("Ripley's K")) {
						for (j = 0; j < oSettings.GetNumberOfFloats(); j++) {
							oMember = (DataMember) oSettings.GetFloat(j);
							bFound = false;
							for (k = 0; k < p_oDataMembers.length; k++) {
								if (p_oDataMembers[k].equals(oMember)) {
									bFound = true; break;
								}
							}
							if (bFound == false) {
								oSettings.RemoveFloat(j);
								j--;
							}
						}
					}
				}
			}
		}
		
    //Validate Condit's Omega calculator parameters
		oBeh = GetBehaviorByKey("Condit's Omega");
		if (oBeh.m_bIsEnabled) {
			MakeSureGreaterThan(m_fConditsOmegaDistanceInc, 0);
			MakeSureGreaterThan(m_fConditsOmegaMaxDistance, 0);
			if (m_fConditsOmegaDistanceInc.GetValue() > m_fConditsOmegaMaxDistance.GetValue()) {
				throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA", "The value for "
						+ m_fConditsOmegaMaxDistance.GetDescriptor()
						+ " must be greater than the value for "
						+ m_fConditsOmegaDistanceInc.GetDescriptor() + "."));
			}

			//Now see if we need to change the number of Relative Neighborhood 
			//Density grid data members since it depends on changes the user may have
			//made to parameters
			int iNumIncs = (int) java.lang.Math.ceil(
					           m_fConditsOmegaMaxDistance.GetValue()	/ 
					           m_fConditsOmegaDistanceInc.GetValue()), 
					iNumSpecies = oPop.GetNumberOfSpecies(), iStart, i, j, k;
			Grid oGrid = m_oManager.GetGridByName("Relative Neighborhood Density");
			DataMember[] p_oDataMembers = oGrid.GetDataMembers();
			if (p_oDataMembers.length != ((iNumIncs * iNumSpecies) + 2)) {
				p_oDataMembers = new DataMember[(iNumIncs * iNumSpecies)+2];
				p_oDataMembers[0] = new DataMember("Distance Inc", "inc", DataMember.FLOAT);
				p_oDataMembers[1] = new DataMember("Max Distance", "dist", DataMember.FLOAT);
				
				iStart = 2;
				for (i = 0; i < iNumSpecies; i++) {
					for (j = 0; j < iNumIncs; j++) {
						p_oDataMembers[iStart] = new DataMember(oPop.GetSpeciesNameFromCode(i)
								.replace('_', ' ')
								+ " Dist " + j + " Omega", j + "_" + i, DataMember.FLOAT);
						iStart++;
					}
				}
				
				oGrid.SetDataMembers(p_oDataMembers);
				
				//Go to the output behavior - if Relative Neighborhood Density is a 
				//grid, erase any data members that may have been removed
				OutputBehaviors oOutput = m_oManager.GetOutputBehaviors();
				DetailedGridSettings oSettings;
				DataMember oMember;
				boolean bFound;
				for (i = 0; i < oOutput.mp_oDetailedGridSaveSettings.size(); i++) {
					oSettings = (DetailedGridSettings) oOutput.mp_oDetailedGridSaveSettings.elementAt(i);
					if (oSettings.GetName().equals("Relative Neighborhood Density")) {
						for (j = 0; j < oSettings.GetNumberOfFloats(); j++) {
							oMember = (DataMember) oSettings.GetFloat(j);
							bFound = false;
							for (k = 0; k < p_oDataMembers.length; k++) {
								if (p_oDataMembers[k].equals(oMember)) {
									bFound = true; break;
								}
							}
							if (bFound == false) {
								oSettings.RemoveFloat(j);
								j--;
							}
						}
					}
				}
			}
		}
	}
}

